@@ -31,7 +31,7 @@
 #define PRINTERVAL 20 /** Print Progress Interval (seconds) */
 #define CRASHFILE "savedstate.sav"
 #define VERSION_MAJOR 0
-#define VERSION_MINOR 11
+#define VERSION_MINOR 12
 
 /** alarmInterrupt is used to print out the progress at specific intervals */
 static void
@@ -251,10 +251,11 @@ main(int argc, char** argv) {
     if(ret) {
       if(ret == EENCNF) 
 	fprintf(stderr, "Error: Could not extract encryption information\n");
-      else if(ret == ETRANF || ret == ETRENF || ret == ETRINF)
+      else if(ret == ETRANF || ret == ETRENF || ret == ETRINF) {
 	fprintf(stderr, "Error: Encryption not detected (is the document password protected?)\n");
-      ret = 4;
-      goto out1;
+	ret = 4;
+	goto out1;
+      }
     }
     else if(e->revision < 2 || (strcmp(e->s_handler,"Standard") != 0)) {
       fprintf(stderr, "The specific version is not supported (%s - %d)\n", e->s_handler, e->revision);
