@@ -108,7 +108,7 @@ initEncKeyWorkSpace(const int revision, const bool encMetaData,
    *   fileID          | <fileIDLEn>
    *  [extra padding]  | [4] (Special for step 6)
    **/
-  unsigned int size = (revision > 3 && !encMetaData) ? 72 : 68;
+  unsigned int size = (revision >= 3 && !encMetaData) ? 72 : 68;
   encKeyWorkSpace = malloc(size + fileIDLen);
 
   /** Just to be sure we have no uninitalized stuff in the workspace */
@@ -127,7 +127,7 @@ initEncKeyWorkSpace(const int revision, const bool encMetaData,
   memcpy(encKeyWorkSpace + 68, fileID, fileIDLen);
 
   /** 6 */
-  if(revision > 3 && !encMetaData) {
+  if(revision >= 3 && !encMetaData) {
     encKeyWorkSpace[68+fileIDLen] = 0xff;
     encKeyWorkSpace[69+fileIDLen] = 0xff;
     encKeyWorkSpace[70+fileIDLen] = 0xff;
@@ -254,19 +254,20 @@ isUserPasswordRev3(void) {
   unsigned int length, j;
 
   length = encdata->length/8;
-  md5(encKeyWorkSpace, ekwlen, enckey);      
+  md5(encKeyWorkSpace, ekwlen, enckey);
   md5_50(enckey);
-  memcpy(test, encdata->u_string, 16);
 
+  memcpy(test, encdata->u_string, PARTIAL_TEST_SIZE);
+
+  /** Algorithm 3.5 reversed */
   RC4_DECRYPT_REV3(PARTIAL_TEST_SIZE);
-      
+
   /** if partial test succeeds we make a full check to be sure */
   if(unlikely(memcmp(test, rev3TestKey, PARTIAL_TEST_SIZE) == 0)) {
     memcpy(test, encdata->u_string, 16);
     RC4_DECRYPT_REV3(16);
-    if(memcmp(test, rev3TestKey, 16) == 0) {
+    if(memcmp(test, rev3TestKey, 16) == 0)
       return true;
-    }
   }
   return false;
 }
@@ -321,7 +322,7 @@ runCrackRev3_o(void) {
 
     do {
       md5(currPW, 32, enckey);
-      
+
       md5_50(enckey);
 
       memcpy(test, encdata->o_string, 32);
@@ -373,13 +374,13 @@ runCrackRev3_of(void) {
   startTime = time(NULL);
   do {
     BEGIN_CRACK_LOOP();
-    
+
     do {
       md5(encKeyWorkSpace, 32, enckey);
-      
+
       md5_50(enckey);
 
-      memcpy(test, encdata->o_string, 32);
+      memcpy(test, encdata->o_string, PARTIAL_TEST_SIZE);
       RC4_DECRYPT_REV3(PARTIAL_TEST_SIZE);
 
       /** if partial test succeeds we make a full check to be sure */
@@ -404,30 +405,59 @@ runCrackRev3(void) {
 
   length = encdata->length/8;
   lpasslength = 0;
-  startTime = time(NULL);
-  do {
-    BEGIN_CRACK_LOOP();
-
+  if(length == 16) {
+    startTime = time(NULL);
     do {
-      md5(encKeyWorkSpace, ekwlen, enckey);
+      BEGIN_CRACK_LOOP();
 
-      md5_50(enckey);
-      memcpy(test, encdata->u_string, 16);
- 
-     /** Algorithm 3.5 reversed */  
-      RC4_DECRYPT_REV3(PARTIAL_TEST_SIZE);
+      do {
+	md5(encKeyWorkSpace, ekwlen, enckey);
 
-      /** if partial test succeeds we make a full check to be sure */
-      if(unlikely(memcmp(test, rev3TestKey, PARTIAL_TEST_SIZE) == 0)) {
-	memcpy(test, encdata->u_string, 16);
-	RC4_DECRYPT_REV3(16);
-	if(memcmp(test, rev3TestKey, 16) == 0)
-	  return true;
-      }
+	md5_50(enckey);
+	memcpy(test, encdata->u_string, PARTIAL_TEST_SIZE);
 
-      ++nrprocessed;
-    } while(permutate());
-  } while(nextPassword());
+	/** Algorithm 3.5 reversed */
+	RC4_DECRYPT_REV3(PARTIAL_TEST_SIZE);
+
+	/** if partial test succeeds we make a full check to be sure */
+	if(unlikely(memcmp(test, rev3TestKey, PARTIAL_TEST_SIZE) == 0)) {
+	  memcpy(test, encdata->u_string, 16);
+	  RC4_DECRYPT_REV3(16);
+	  if(memcmp(test, rev3TestKey, 16) == 0)
+	    return true;
+	}
+
+	++nrprocessed;
+      } while(permutate());
+    } while(nextPassword());
+  }
+  else {
+    startTime = time(NULL);
+    do {
+      BEGIN_CRACK_LOOP();
+
+      do {
+	md5(encKeyWorkSpace, ekwlen, enckey);
+
+	for (i=0; i < 50; i++) { md5(enckey, length, enckey); }
+
+	memcpy(test, encdata->u_string, PARTIAL_TEST_SIZE);
+
+	/** Algorithm 3.5 reversed */
+	RC4_DECRYPT_REV3(PARTIAL_TEST_SIZE);
+
+	/** if partial test succeeds we make a full check to be sure */
+	if(unlikely(memcmp(test, rev3TestKey, PARTIAL_TEST_SIZE) == 0)) {
+	  memcpy(test, encdata->u_string, length);
+	  RC4_DECRYPT_REV3(length);
+	  if(memcmp(test, rev3TestKey, length) == 0)
+	    return true;
+	}
+
+	++nrprocessed;
+      } while(permutate());
+    } while(nextPassword());
+  }
   return false;
 }
 
@@ -546,7 +576,10 @@ initPDFCrack(const EncData *e, const uint8_t *upw, const bool user,
   nrprocessed = 0;
   workWithUser = user;
   crackDone = false;
-  setrc4DecryptMethod((const unsigned int)e->length);
+
+  if(!setrc4DecryptMethod((const unsigned int)e->length))
+    exit(234);
+
   if(upw) {
     upwlen = strlen((const char*)upw);
     if(upwlen > 32)
@@ -614,7 +647,8 @@ static const char string_UUPWP[] =
 bool
 loadState(FILE *file, EncData *e, char **wl, bool *user) {
   unsigned int i;
-  int tmp, tmp2, tmp3, len;
+  int tmp, tmp2, tmp3;
+  size_t len;
   char c;
 
   /** Load all the simple values bound to the document */
