@@ -330,6 +330,120 @@ parseRegularString(FILE *file) {
 return ret;
 }
 
+static int
+findTrailerDict(FILE *file, EncData *e) {
+  int ch;
+  /**  int pos_i; */
+  bool encrypt = false;
+  bool id = false;
+  int e_pos = -1;
+  p_str *str = NULL;
+  int dict = 0;
+  
+  ch = getc(file);
+  while(ch != EOF) {
+    if(isEndOfLine(ch)) {
+      ch = parseWhiteSpace(file);
+      if(ch == '<' && getc(file) == '<') {
+	/** This should be the first dict we stumble upon*/
+
+	/**
+	   pos_i = ftell(file);
+	   printf("found Trailer at pos %x\n", pos_i);
+	*/
+	ch = getc(file);
+	while(ch != EOF) {
+	  if(ch == '<') {
+	    ch = getc(file);
+	    if(ch == '<') {
+	      dict++;
+	    }
+	  }
+	  if(ch == '>') {
+	    ch = getc(file);
+	    if(ch == '>') {
+	      if(dict == 0)
+		break;
+	    }
+	    dict--;
+	  }
+	  while(ch != '/' && ch != EOF) {
+	    ch = getc(file);
+	  }
+	  ch = getc(file);
+	  /**printf("found a name: %c\n", ch);*/
+	  if(e_pos < 0 && ch == 'E' && isWord(file, "ncrypt")) {
+	    e_pos = parseIntWithC(file,parseWhiteSpace(file));
+	    if(e_pos >= 0) {
+	      /**
+		 pos_i = ftell(file);
+		 printf("found Encrypt at pos %x, ", pos_i);
+		 printf("%d\n", e_pos);
+	      */
+	      encrypt = true;
+	    }
+	  }
+	  else if(ch == 'I' && getc(file) == 'D') {
+	    ch = parseWhiteSpace(file);
+	    while(ch != '[' && ch != EOF)
+	      ch = getc(file);
+
+	    if(str) {
+	      if(str->content)
+		free(str->content);
+	      free(str);
+	    }
+	      
+	    str = parseRegularString(file);
+	    /**
+	       pos_i = ftell(file);
+	       printf("found ID at pos %x\n", pos_i);
+	    */
+	    if(str)
+	      id = true;
+	    ch = getc(file);
+	  }
+	  else
+	    ch = getc(file);
+	  if(encrypt && id) {
+	    /**printf("found all, returning: epos: %d\n",e_pos);*/
+	    e->fileID = str->content;
+	    e->fileIDLen = str->len;
+	    free(str);
+	    return e_pos;
+	  }
+	}
+      }  
+      else {
+	ch = getc(file);
+      }     
+    }
+    else
+      ch = getc(file);
+  }
+  /**  printf("finished searching\n");*/
+  
+  if(str) {
+    if(str->content)
+      	free(str->content);
+    free(str);
+  }
+  
+  if(!encrypt && id)
+    return ETRENF;
+  else if(!id && encrypt) {
+    /** We found a encrypt object, but not an ID. Let us try parsing the 
+	object as some security handlers seems to encrypt/skip the ID.
+	Logic changed to show these handles correctly instead of printing out
+	a error that the document is not encrypted.
+    **/
+    return e_pos;
+  }
+  else 
+    return ETRANF;
+}
+
+
 static int
 findTrailer(FILE *file, EncData *e) {
   int ch;
@@ -425,8 +539,14 @@ findTrailer(FILE *file, EncData *e) {
 
   if(!encrypt && id)
       return ETRENF;
-  else if(!id && encrypt)
-    return ETRINF;
+  else if(!id && encrypt) {
+    /** We found a encrypt object, but not an ID. Let us try parsing the 
+	object as some security handlers seems to encrypt/skip the ID.
+	Logic changed to show these handles correctly instead of printing out
+	a error that the document is not encrypted.
+    **/
+   return e_pos;
+  }
   else 
     return ETRANF;
 }
@@ -442,6 +562,8 @@ parseEncrypObject(FILE *file, EncData *e) {
   bool fr = false;
   bool fu = false;
   bool fv = false;
+  bool cf = false;
+  bool aesv2 = false;
   p_str *str = NULL;
 
   ch = getc(file);
@@ -460,9 +582,32 @@ parseEncrypObject(FILE *file, EncData *e) {
 	dict++;
       }
     }
+    if(dict > 1) {
+      ch = getc(file);
+      if(ch == '/') {
+	ch = getc(file);
+	switch(ch) {
+	case 'A':
+	  if(isWord(file, "ESV2")) {
+	    aesv2 = true;
+	  }
+	  break;
+
+	default:
+	  break;
+	}
+      }
+      continue;
+    }
+
     if(ch == '/') {
       ch = getc(file);
       switch(ch) {
+      case 'C':
+	if(isWord(file, "F")) {
+	  cf = true;
+	}
+	break;	
       case 'E':
 	if(isWord(file, "ncryptMetadata")) {
 	  ungetc(parseWhiteSpace(file), file);
@@ -485,6 +630,9 @@ parseEncrypObject(FILE *file, EncData *e) {
 	  if(!fl) { 
 	    /* BZZZT!!  This is sooo wrong but will work for most cases.
 	       only use the first length we stumble upon */
+	    if(tmp_l > 128 && tmp_l < 40 && ((tmp_l % 8) != 0)) {
+	      fprintf(stderr, "WARNING: Length = %d\n", tmp_l);
+	    }
 	    e->length = tmp_l;
 	  }
 	  fl = true;
@@ -539,6 +687,7 @@ parseEncrypObject(FILE *file, EncData *e) {
     }
     ch = parseWhiteSpace(file);
   }
+    printf("\n");
 
   if(!fe)
     e->encryptMetaData = true;
@@ -547,6 +696,11 @@ parseEncrypObject(FILE *file, EncData *e) {
   if(!fv)
     e->version = 0;
 
+  if(fr && cf && aesv2) {
+    e->revision = 3;
+    e->version = 2;
+  }
+
   if(strcmp(e->s_handler,"Standard") != 0)
     return true;
 
@@ -560,7 +714,6 @@ parseEncrypObject(FILE *file, EncData *e) {
 static bool
 findEncryptObject(FILE *file, const int e_pos, EncData *e) {
   int ch;
-  int pos_i;
 
   /** only find the encrypt object if e_pos > -1 */
   if(e_pos < 0)
@@ -575,7 +728,7 @@ findEncryptObject(FILE *file, const int e_pos, EncData *e) {
 	  ch = parseWhiteSpace(file);
 	  if(ch == 'o' && getc(file) == 'b' && getc(file) == 'j' &&
 	     parseWhiteSpace(file) == '<' && getc(file) == '<') {
-	    pos_i = ftell(file);
+	    ftell(file);
 	    return parseEncrypObject(file, e);
 	  }
 	}
@@ -598,6 +751,11 @@ getEncryptedInfo(FILE *file, EncData *e) {
     rewind(file);
     e_pos = findTrailer(file, e);
   }
+  if(e_pos < 0) {
+    rewind(file);
+    e_pos = findTrailerDict(file, e);	
+  }
+
   if(e_pos < 0) {
     return e_pos;
   }
